@@ -7,25 +7,27 @@
 #include "qmlobject.h"
 #include "private/kdeclarative_p.h"
 
-#include <QQmlEngine>
 #include <QQmlContext>
-#include <QQuickItem>
+#include <QQmlEngine>
 #include <QQmlIncubator>
+#include <QQmlNetworkAccessManagerFactory>
+#include <QQuickItem>
 #include <QTimer>
 
+#include <KPackage/PackageLoader>
 #include <QDebug>
 #include <kdeclarative.h>
-#include <KPackage/PackageLoader>
 
 //#include "packageaccessmanagerfactory.h"
 //#include "private/declarative/dataenginebindings_p.h"
 
-namespace KDeclarative {
-
+namespace KDeclarative
+{
 class QmlObjectIncubator : public QQmlIncubator
 {
 public:
     QVariantHash m_initialProperties;
+
 protected:
     void setInitialState(QObject *object) override
     {
@@ -41,15 +43,17 @@ class QmlObjectPrivate
 {
 public:
     QmlObjectPrivate(QmlObject *parent)
-        : q(parent),
-          engine(nullptr),
-          component(nullptr),
-          delay(false)
+        : q(parent)
+        , engine(nullptr)
+        , component(nullptr)
+        , delay(false)
     {
         executionEndTimer = new QTimer(q);
         executionEndTimer->setInterval(0);
         executionEndTimer->setSingleShot(true);
-        QObject::connect(executionEndTimer, SIGNAL(timeout()), q, SLOT(scheduleExecutionEnd()));
+        QObject::connect(executionEndTimer, &QTimer::timeout, q, [this]() {
+            scheduleExecutionEnd();
+        });
     }
 
     ~QmlObjectPrivate()
@@ -71,12 +75,12 @@ public:
     QmlObject *q;
 
     QUrl source;
-    QQmlEngine *engine;
+    std::shared_ptr<QQmlEngine> engine;
+
     QmlObjectIncubator incubator;
     QQmlComponent *component;
     QTimer *executionEndTimer;
-    KDeclarative kdeclarative;
-    KLocalizedContext *context{ nullptr };
+    KLocalizedContext *context{nullptr};
     KPackage::Package package;
     QQmlContext *rootContext;
     bool delay : 1;
@@ -88,8 +92,8 @@ void QmlObjectPrivate::errorPrint(QQmlComponent *component)
     if (component->isError()) {
         const QList<QQmlError> errors = component->errors();
         for (const QQmlError &error : errors) {
-            errorStr += (error.line() > 0 ? QString(QString::number(error.line()) + QLatin1String(": ")) : QLatin1String(""))
-                        + error.description() + QLatin1Char('\n');
+            errorStr +=
+                (error.line() > 0 ? QString(QString::number(error.line()) + QLatin1String(": ")) : QLatin1String("")) + error.description() + QLatin1Char('\n');
         }
     }
     qWarning() << component->url().toString() << '\n' << errorStr;
@@ -103,9 +107,8 @@ void QmlObjectPrivate::execute(const QUrl &source)
     }
 
     delete component;
-    component = new QQmlComponent(engine, q);
-    QObject::connect(component, &QQmlComponent::statusChanged,
-                     q, &QmlObject::statusChanged, Qt::QueuedConnection);
+    component = new QQmlComponent(engine.get(), q);
+    QObject::connect(component, &QQmlComponent::statusChanged, q, &QmlObject::statusChanged, Qt::QueuedConnection);
     delete incubator.object();
 
     component->loadUrl(source);
@@ -122,46 +125,47 @@ void QmlObjectPrivate::scheduleExecutionEnd()
     if (component->isReady() || component->isError()) {
         q->completeInitialization();
     } else {
-        QObject::connect(component, SIGNAL(statusChanged(QQmlComponent::Status)), q, SLOT(completeInitialization()));
+        QObject::connect(component, &QQmlComponent::statusChanged, q, [this]() {
+            q->completeInitialization();
+        });
     }
 }
 
 QmlObject::QmlObject(QObject *parent)
-    // cannot do : QmlObject(new QQmlEngine(this), d->engine->rootContext(), parent)
-    : QObject(parent)
-    , d(new QmlObjectPrivate(this))
-
+    : QmlObject(nullptr, nullptr, parent)
 {
-    d->engine = new QQmlEngine(this);
-    d->rootContext = d->engine->rootContext();
-    d->kdeclarative.setDeclarativeEngine(d->engine);
-    d->kdeclarative.d->qmlObj = this;
-
-    d->context = new KLocalizedContext(this);
-    d->rootContext->setContextObject(d->context);
-    KDeclarative::setupEngine(d->engine);
 }
 
+#if KDECLARATIVE_BUILD_DEPRECATED_SINCE(5, 95)
 QmlObject::QmlObject(QQmlEngine *engine, QObject *parent)
-    : QmlObject(engine, engine->rootContext(), parent)
+    : QmlObject(std::shared_ptr<QQmlEngine>(engine), nullptr, parent)
 {
-
 }
 
 QmlObject::QmlObject(QQmlEngine *engine, QQmlContext *rootContext, QObject *parent)
-    : QmlObject(engine, rootContext, nullptr /*call setupEngine*/, parent)
+    : QmlObject(std::shared_ptr<QQmlEngine>(engine), rootContext, parent)
 {
-
 }
 
 QmlObject::QmlObject(QQmlEngine *engine, QQmlContext *rootContext, QmlObject *obj, QObject *parent)
+    : QmlObject(std::shared_ptr<QQmlEngine>(engine), rootContext, parent)
+{
+    Q_UNUSED(obj);
+}
+#endif
+
+QmlObject::QmlObject(std::shared_ptr<QQmlEngine> engine, QQmlContext *rootContext, QObject *parent)
     : QObject(parent)
     , d(new QmlObjectPrivate(this))
 {
     if (engine) {
         d->engine = engine;
     } else {
-        d->engine = new QQmlEngine(this);
+        d->engine = std::make_shared<QQmlEngine>();
+    }
+
+    if (d->engine.use_count() <= 2) {
+        KDeclarative::setupEngine(d->engine.get());
     }
 
     if (rootContext) {
@@ -169,22 +173,23 @@ QmlObject::QmlObject(QQmlEngine *engine, QQmlContext *rootContext, QmlObject *ob
     } else {
         d->rootContext = d->engine->rootContext();
     }
-    d->kdeclarative.setDeclarativeEngine(d->engine);
-    d->kdeclarative.d->qmlObj = this;
 
     d->context = new KLocalizedContext(this);
     d->rootContext->setContextObject(d->context);
-
-    if (!obj) {
-        KDeclarative::setupEngine(d->engine);
-    }
 }
 
 QmlObject::~QmlObject()
 {
-//    QDeclarativeNetworkAccessManagerFactory *factory = d->engine->networkAccessManagerFactory();
-//    d->engine->setNetworkAccessManagerFactory(0);
-//    delete factory;
+    if (d->engine.use_count() == 1) {
+        // QQmlEngine does not take ownership of the QNAM factory so we need to
+        // make sure to clean it, but only if we are the last user of the engine
+        // otherwise we risk resetting the factory on an engine that is still in
+        // use.
+        auto factory = d->engine->networkAccessManagerFactory();
+        d->engine->setNetworkAccessManagerFactory(nullptr);
+        delete factory;
+    }
+
     delete d;
 }
 
@@ -239,7 +244,7 @@ bool QmlObject::isInitializationDelayed() const
 
 QQmlEngine *QmlObject::engine()
 {
-    return d->engine;
+    return d->engine.get();
 }
 
 QObject *QmlObject::rootObject() const
@@ -285,7 +290,7 @@ void QmlObjectPrivate::checkInitializationCompleted()
         errorPrint(component);
     }
 
-    emit q->finished();
+    Q_EMIT q->finished();
 }
 
 void QmlObject::completeInitialization(const QVariantHash &initialProperties)
@@ -300,7 +305,6 @@ void QmlObject::completeInitialization(const QVariantHash &initialProperties)
         return;
     }
 
-
     if (d->component->status() != QQmlComponent::Ready || d->component->isError()) {
         d->errorPrint(d->component);
         return;
@@ -317,13 +321,13 @@ void QmlObject::completeInitialization(const QVariantHash &initialProperties)
         if (!d->incubator.object()) {
             d->errorPrint(d->component);
         }
-        emit finished();
+        Q_EMIT finished();
     }
 }
 
 QObject *QmlObject::createObjectFromSource(const QUrl &source, QQmlContext *context, const QVariantHash &initialProperties)
 {
-    QQmlComponent *component = new QQmlComponent(d->engine, this);
+    QQmlComponent *component = new QQmlComponent(d->engine.get(), this);
     component->loadUrl(source);
 
     return createObjectFromComponent(component, context, initialProperties);
@@ -335,13 +339,13 @@ QObject *QmlObject::createObjectFromComponent(QQmlComponent *component, QQmlCont
     incubator.m_initialProperties = initialProperties;
     component->create(incubator, context ? context : d->rootContext);
     incubator.forceCompletion();
-  
+
     QObject *object = incubator.object();
 
     if (!component->isError() && object) {
-        //memory management
+        // memory management
         component->setParent(object);
-        //reparent to root object if wasn't specified otherwise by initialProperties
+        // reparent to root object if wasn't specified otherwise by initialProperties
         if (!initialProperties.contains(QLatin1String("parent"))) {
             if (qobject_cast<QQuickItem *>(rootObject())) {
                 object->setProperty("parent", QVariant::fromValue(rootObject()));
